#ifndef __FAMILYLIKELIHOODLG_H__
#define __FAMILYLIKELIHOODLG_H__

#include "PedMarkerData.h"
#include "Inheritance_Matrix.h"
#include "MathMatrix.h"
#include <algorithm>
#include <omp.h>

class FamilyLikelihoodLG
{
 public:
  PedMarkerData *pm;
  PedMarkerData pmd;
  Inheritance_Matrix *im;
  float *recomb;
  std::vector<float> trans;
  SeqMarkerInfo** fam_markers;
  std::vector<int> pow2;
  int fam_idx;
  int marker_count;
  int ac;
  int dp;

  float **pvector;
  float **lvector;
  float **rvector;
  float **delta;
  uint32_t **phi;
  std::vector<int> best_path;
  std::vector<int> best_marginal;
  std::vector<float> scaling_factors;
  std::vector<float> geno_prior;
  std::vector<float> lk_single_point;

  Matrix geno_posterior;
  std::vector<int> best_geno_idx;
  std::vector<std::string> best_geno_label;
  std::vector<int> geno_qual;
  std::vector<float> dosage;
  std::vector<std::string> geno_labels;

  std::vector<int> best_phased_geno_idx;
  std::vector<std::string> best_phased_geno_label;
  float phased_geno_qual;
  std::vector<std::string> phased_geno_labels;

  double cumulative_marginals;
  float **marginals;
  SeqMarkerInfo **tmpMarkers;

  bool onfly;
  std::vector<std::vector<uint8_t> > geno_index_tmp;

  double *DblTmpVector;
  std::vector<float> tmpMarginals;
  
 public:
  FamilyLikelihoodLG();
  ~FamilyLikelihoodLG();
  int CompareDouble(const double * a, const double * b);
  void Initialize();
  void Clear();
  void Clear_Partial();
  void DeleteMemory();
  void SetOnflyFlag(bool flag) { onfly = flag; }
  void SetRecomb(float *r) { recomb = r; }
  void SetPedMarkerData(PedMarkerData*, int fam_idx);
  void SetInheritanceMatrix(Inheritance_Matrix*);
  void SetGenoPrior(float&);
  void SetCumulativeMarginals(double c) { cumulative_marginals = c; }
  void MaskMarginals(int);
  void AllocateMemory();
  float CalculateRefLogLikelihood(SeqMarkerInfo *markers);
  float CalculateSingleVectorLikelihood(int, int);
  float CalculateSingleVectorLikelihood(int, SeqMarkerInfo*);
  float CalculateSingleVectorLikelihood(int, std::vector<std::vector<uint8_t> >& geno_index_onfly, int);
  float CalculateSingleVectorLikelihood(int, std::vector<std::vector<uint8_t> >& geno_index_onfly, SeqMarkerInfo*);
  float CalculateSingleFounderAllelesLikelihood(int, int);
  void CalculateIVProb();
  float CalculateSinglePointLikelihood(int marker_inex);
  float CalculateSinglePointLikelihood_IBD(SeqMarkerInfo *, float*);
  float CalculateSinglePointLikelihood_IBD(int);
  float CalculateSinglePointLikelihood_IBD(SeqMarkerInfo *, float*, int);
  float CalculateSinglePointLikelihood_IBD(int, int);
  float CalculateSinglePointLikelihood_IBD_Geno(SeqMarkerInfo*, float*, int, int, int, int, bool);
  float CalculateSinglePointLikelihood_IBD_Geno(int, int, int, int, int);
  float GetSinglePointLikelihood(int marker_inex);
  void CalculateMultiPointLikelihood_forward();
  void CalculateMultiPointLikelihood_backward();
  void CalculateMultiPointLikelihood_forward_FFT();
  void CalculateMultiPointLikelihood_backward_FFT();
  void CalculateMarginals();
  void CalculateGenoPosteriors(SeqMarkerInfo*, float*, int max_idx, bool single_marginal);
  int CalculateSingleVectorMaxHapLikelihood(int iv_index, SeqMarkerInfo *markers);
  void CalculatePhasedGeno_best(SeqMarkerInfo *markers, int);
  void CalculatePhasedGeno_marginal(SeqMarkerInfo *markers, float *marginals);
  void FFT(float *);
  void Convolve_FFT(float, float*);
  float Scale(float *);
  float Scale(double *, float *);
  void Scale(float *, float);
  void FindBestMarginal();
  void FindBestPath();
  void PrintBestPath();
  void PrintForwardProb();
  void PrintBackwardProb();
  void PrintMarginalProb();
  void quickSort(float arr[], int left, int right);
};

#endif
