#include "FamilyLikelihoodLG.h"
#include "PedSeq.h"
#include "Parameters.h"
#include "StringArray.h"
#include "Error.h"
#include <iostream>

using namespace std;

int main(int argc, char* argv[])
{
  String pedFile, datFile, mapFile;
  String out_chr_range, in_chr_range, range_list_file;
  String inVCF;
  String outVCF;
  bool output_all = false;
  bool disable_fft = true;
  bool phase_marginal = false;
  bool phase_best = false;
  bool single_marginal = false;
  bool onfly = false;
  bool mixed_vcf_records = false;
  bool trio_only = false;

  double theta = 0.001;
  double theta_indel = 0.0001;
  double tstv_ratio = 2.0;
 
  double cumulative_marginals = 1.0;
  double submap = 1.0;

  int nthreads=1;
  
  std::string in_chr, out_chr;
  int in_start, out_start;
  int in_end, out_end;
  int famIdx = 0;
  String famID;

  std::string cmd;
  for(int a=0; a<argc; a++)
  {
   cmd += std::string(argv[a]);
   cmd += " ";
  }

  std::string chr_prefix = "";

  ParameterList pl;

  BEGIN_LONG_PARAMETERS(longParameters)
    LONG_PARAMETER_GROUP("Input")
    LONG_STRINGPARAMETER("in_vcf", &inVCF)
    LONG_STRINGPARAMETER("in_range", &in_chr_range)
    LONG_PARAMETER("mixed_vcf_records", &mixed_vcf_records)
//    LONG_PARAMETER("trio_only", &trio_only)
    LONG_PARAMETER_GROUP("Mutation paramters")
    LONG_DOUBLEPARAMETER("theta_snv", &theta)
    LONG_DOUBLEPARAMETER("theta_indel", &theta_indel)
    LONG_DOUBLEPARAMETER("tstv_ratio", &tstv_ratio)
    LONG_DOUBLEPARAMETER("submap", &submap)

    LONG_PARAMETER_GROUP("Multi-threading")
    LONG_INTPARAMETER("nthreads", &nthreads)
//    LONG_PARAMETER("onfly", &onfly)
    LONG_PARAMETER_GROUP("Output")
    LONG_STRINGPARAMETER("out_vcf", &outVCF)
    LONG_INTPARAMETER("fam_idx", &famIdx)
    LONG_STRINGPARAMETER("fam_id", &famID)
    LONG_PARAMETER("out_all", &output_all)
    LONG_STRINGPARAMETER("out_range", &out_chr_range)
    //LONG_PARAMETER("disable_fft", &disable_fft)
    LONG_PARAMETER("best_marginal", &phase_marginal)
    LONG_PARAMETER("best_path", &phase_best)
    LONG_PARAMETER_GROUP("Approximation")
    LONG_DOUBLEPARAMETER("cum_prob", &cumulative_marginals)
    LONG_PARAMETER("single_iv", &single_marginal)
    END_LONG_PARAMETERS();
  
  pl.Add(new StringParameter('p', "pedfile", pedFile));
  pl.Add(new StringParameter('d', "datfile", datFile));
  pl.Add(new StringParameter('m', "mapfile", mapFile));
  pl.Add(new LongParameters("Additional Options", longParameters));
  
  pl.Read(argc, argv);
  
  pl.Status();

  if(pedFile.Length()==0)
    error("pedFile not provided for input!\n");
  if(mapFile.Length()==0)
    error("mapFile not provided for input!\n");
  if(inVCF.Length()==0)
    error("Input VCF file not provided for input!\n");
  if(outVCF.Length()==0)
    error("Output VCF file not provided for input!\n");

  if(inVCF == outVCF) error("Input and output VCF files are the same!\n");

  IFILE pedFH = ifopen(pedFile, "r");
  IFILE vcfFH = ifopen(inVCF, "r");

  IFILE mapFH = NULL;
  if(trio_only==false) 
  {
    mapFH = ifopen(mapFile, "r");
    if(mapFH==NULL)
      error("mapFile open for input failed!\n");
  }

  IFILE datFH = NULL;
  if(datFile.Length()>0)
  {
    datFH = ifopen(datFile, "r");
    if(datFH==NULL)
     error("datFile open for input failed!\n");
  }

  if(pedFH==NULL)
    error("pedFile open for input failed!\n");
  if(vcfFH==NULL)
    error("Input VCF file open failed!\n");

  #ifdef _OPENMP
  if(nthreads>0) omp_set_num_threads(nthreads);
  #endif

  if(in_chr_range.Length()>0)
    {
      StringArray tokens;
      tokens.ReplaceTokens(in_chr_range, ":");
      if(tokens.Length()!=2) error("Chr range should be in the format of chrName:start-end or chrName:0 to include the entire chrName\n");
      in_chr = string(tokens[0].c_str() );
      if(in_chr.compare("chr")==0) { in_chr = in_chr.substr(3, in_chr.size()); chr_prefix = "chr"; }

      StringArray tokens2;
      tokens2.ReplaceTokens(tokens[1], "-");
      if(tokens2.Length()!=2) {
	std::cout<<"Entire chromosome "<<in_chr<<" is included for input"<<endl;
	in_start = 0; in_end = 0;
      }
      else {
	in_start = tokens2[0].AsInteger();
	in_end = tokens2[1].AsInteger();
      }

      if(in_start>in_end) error("Input range is not correct: %s\n", in_chr_range.c_str() );
    }

  if(out_chr_range.Length()>0)
    {
      StringArray tokens;
      tokens.ReplaceTokens(out_chr_range, ":");
      if(tokens.Length()!=2) error("Chr range should be in the format of chrName:start-end or chrName:0 to include the entire chrName\n");
      out_chr = string(tokens[0].c_str() );
      if(out_chr.compare("chr")==0) { out_chr = out_chr.substr(3, out_chr.size()); chr_prefix="chr"; }

      StringArray tokens2;
      tokens2.ReplaceTokens(tokens[1], "-");
      if(tokens2.Length()!=2) {
	std::cout<<"Entire chromosome "<<out_chr<<" is included to output to VCF"<<endl;
	out_start = 0; out_end = 0;
      }
      else {
	out_start = tokens2[0].AsInteger();
	out_end = tokens2[1].AsInteger();
      }
     if(out_start>out_end) error("Output range is not correct: %s\n", out_chr_range.c_str() );

     if(in_chr_range.Length()>0 && in_chr.compare(out_chr)!=0) error("Input and output chromosomes are different!\n");
     if(in_chr_range.Length()>0 && (in_start>out_start || in_end<out_end && in_end!=0)) error("Input range should include output ranage!\n");
    }


  Pedigree ped;

  if(datFH!=NULL) ped.Prepare(datFH);
  ped.Load(pedFH);

  printf("Loading marker info with submap %f...\n", submap);

  //Load marker info
  PedMarkerData ped_markers;
  ped_markers.SetParam(mixed_vcf_records);
  ped_markers.SetPedigree(&ped);
  ped_markers.SetInputRange(in_chr, in_start, in_end);
  ped_markers.SetSubmap(submap);
  ped_markers.LoadMapFile(mapFile);
  ped_markers.LoadMarkerVCF(inVCF);

  //Genotype calling for all families in a pedigree
  //////////////////////////////////////////////////

  //Set input and output files
  PedSeq ps;
  ps.SetCmd(cmd);
  ps.SetParam(mixed_vcf_records, trio_only);
  ps.SetCumulativeMarginals(cumulative_marginals);
  ps.SetFamilyID(std::string(famID.c_str()));
  ps.SetFamilyIdx(famIdx);
  ps.SetOnflyFlag(onfly);
  ps.SetInputVCFFile(std::string(inVCF.c_str()) );
  ps.SetOutputVCFFile(std::string(outVCF.c_str()) );
  ps.SetOutputRange(out_chr, out_start, out_end);
  ps.SetChrPrefix(chr_prefix);
  ps.SetOutputAllFlag(output_all);
  ps.SetPedMarkerData(&ped_markers);

printf("Setting up inheritance vectors...\n");

  //Set inheritance and marker data for all families
  ps.SetInheritanceMatrix();

  //Set mutation parameters and priors
  ps.SetTsTvRatio(tstv_ratio);
  ps.SetTheta(theta);
  ps.SetTheta_Indel(theta_indel);
  ps.SetPolyPrior();
  ps.SetFFTFlag(disable_fft);
  ps.SetPhaseFlag(phase_marginal, phase_best);
  ps.SetSingleMarginal(single_marginal);
  // Calculate likelihoods and perform genotype calling
  ps.CalculateIVMarginals();
  ps.CalculateGenoPosteriorAndOutput2VCF(); //Reload VCF input file and infer genotypes for all variants

}
